<?php

namespace Drupal\page_metatag\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class MetaTagsSettingsForm.
 */
class PageMetatagSettingsForm extends ConfigFormBase {

  protected $languageManager;
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public function __construct(LanguageManagerInterface $language_manager = NULL, ModuleHandlerInterface $module_handler) {
    if ($language_manager) {
      $this->languageManager = $language_manager;
    }
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $languageServices = NULL;
    if ($container->has('language_manager')) {
      $languageServices = $container->get('language_manager');
    }
    return new static(
      $languageServices,
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'page_metatag_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'page_metatag.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('page_metatag.settings');
    $form['desc'] = [
      '#type' => 'markup',
      '#markup' => $this->t('
        <b>Enabled:</b> Rule will work only if checkbox is checked.<br>
        <b>Pages:</b> Enter one path per line. The "*" character is a wildcard. Example paths are "/node/1" for an individual piece of content or "/node/*" for every piece of content. "@front" is the front page.<br>'),
    ];
    // Headers for table.
    $header = [
      ['data' => $this->t('Enabled'), 'width' => 50],
      ['data' => $this->t('Pages'),'width' => 300],
      ['data' => $this->t('Page Title/Keywords'),'width' => 400],
      ['data' => $this->t('Description'),'width' => 400],
    ];
    if ($this->languageManager->isMultilingual() || $this->moduleHandler->moduleExists('language')) {
      $header[] = $this->t('Language');
    }
    array_push($header, $this->t('Operation'), $this->t('Weight'));

    // Multi value table form.
    $form['page_metatag_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => $this->t('There are no items yet. Add an item.', []),
      '#prefix' => '<div id="spt-fieldset-wrapper">',
      '#suffix' => '</div>',
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'page_metatag_table-order-weight',
        ],
      ],
    ];

    // Set table values on Add/Remove or on page load.
    $page_metatag_table = $form_state->get('page_metatag_table');
    if (empty($page_metatag_table)) {
      // Set data from configuration on page load.
      // Set empty element if no configurations are set.
      if (NULL !== $config->get('page_metatag_table')) {
        $page_metatag_table = $config->get('page_metatag_table');
        $form_state->set('page_metatag_table', $page_metatag_table);
      }
      else {
        $page_metatag_table = [''];
        $form_state->set('page_metatag_table', $page_metatag_table);
      }
    }

    // Create row for table.
    foreach ($page_metatag_table as $i => $value) {
      $form['page_metatag_table'][$i]['#attributes']['class'][] = 'draggable';
      $form['page_metatag_table'][$i]['status'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Status'),
        '#title_display' => 'invisible',
        '#default_value' => isset($value['status']) ? $value['status'] : '',
      ];
      $form['page_metatag_table'][$i]['pages'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Pages'),
        '#title_display' => 'invisible',
        '#required' => TRUE,
        '#cols' => '5',
        '#rows' => '5',
        '#default_value' => isset($value['pages']) ? $value['pages'] : [],
      ];
      $metadata = [
        'page_title' => [
          '#type' => 'textfield',
          '#title' => $this->t('Page Title'),
          '#title_display' => 'invisible',
          '#required' => TRUE,
          '#size' => 10,
          '#maxlength' => 60,
          '#attributes' => ['placeholder' => 'Title'],
          '#default_value' => isset($value['title']['page_title']) ? $value['title']['page_title'] : [],
        ],
        'page_keywords' => [
          '#type' => 'textfield',
          '#title' => $this->t('Page Keywords'),
          '#title_display' => 'invisible',
          '#size' => 10,
          '#maxlength' => 255,
          '#description' => t('Keywords separate by comma.'),
          '#attributes' => ['placeholder' => 'Keywords'],
          '#default_value' => isset($value['title']['page_keywords']) ? $value['title']['page_keywords'] : [],
        ],
      ];
      $form['page_metatag_table'][$i]['title'] = $metadata;
      $form['page_metatag_table'][$i]['description'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Description'),
        '#title_display' => 'invisible',
        '#required' => TRUE,
        '#cols' => '5',
        '#rows' => '5',
        '#maxlength' => 160,
        '#default_value' => isset($value['description']) ? $value['description'] : [],
      ];
      // Add Language if site is multilingual.
      if ($this->languageManager->isMultilingual() || $this->moduleHandler->moduleExists('language')) {
        foreach ($this->languageManager->getLanguages() as $langkey => $langvalue) {
          $langNames[$langkey] = $langvalue->getName();
        }
        $form['page_metatag_table'][$i]['language'] = [
          '#type' => 'checkboxes',
          '#title' => $this->t('Language'),
          '#title_display' => 'invisible',
          '#options' => $langNames,
          '#default_value' => isset($value['language']) ? $value['language'] : [],
        ];
      }
      $form['page_metatag_table'][$i]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#name' => "remove-" . $i,
        '#submit' => ['::removeElement'],
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => '::removeCallback',
          'wrapper' => 'spt-fieldset-wrapper',
        ],
        '#index_position' => $i,
      ];
      // TableDrag: Weight column element.
      $form['page_metatag_table'][$i]['weight'] = [
        '#type' => 'weight',
        '#title_display' => 'invisible',
        '#default_value' => isset($value['weight']) ? $value['weight'] : [],
        '#attributes' => ['class' => ['page_metatag_table-order-weight']],
      ];
    }
    $form['add_name'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add one more'),
      '#submit' => ['::addOne'],
      '#ajax' => [
        'callback' => '::addmoreCallback',
        'wrapper' => 'spt-fieldset-wrapper',
      ],
    ];
    $form['token_key']['token_tree'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['current-page', 'site', 'node'],
      '#show_restricted' => TRUE,
      //'#global_types' => FALSE,
      '#weight' => 1,
    ];

    $form_state->setCached(FALSE);
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function addmoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['page_metatag_table'];
  }

  /**
   * {@inheritdoc}
   */
  public function addOne(array &$form, FormStateInterface $form_state) {
    $page_metatag_table = $form_state->get('page_metatag_table');
    array_push($page_metatag_table, "");
    $form_state->set('page_metatag_table', $page_metatag_table);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    return $form['page_metatag_table'];
  }

  /**
   * {@inheritdoc}
   */
  public function removeElement(array &$form, FormStateInterface $form_state) {
    // Get table.
    $page_metatag_table = $form_state->get('page_metatag_table');
    // Get element to remove.
    $remove = key($form_state->getValue('page_metatag_table'));
    // Remove element.
    unset($page_metatag_table[$remove]);
    // Set an empty element if no elements are left.
    if (empty($page_metatag_table)) {
      array_push($page_metatag_table, "");
    }
    $form_state->set('page_metatag_table', $page_metatag_table);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('page_metatag.settings')->set('page_metatag_table', $form_state->getValue('page_metatag_table'))->save();
    parent::submitForm($form, $form_state);
  }

}
